<?php

require_once("classes/translator.class.php");
tl::register("exchange", dirname(__FILE__)."/lang");

require_once("pbx/pbx.php");
require_once("ews/ExchangeServices.php");
require_once("include_main.php");
require_once("postgresql.php");

class PbxException extends Exception
{
    var $details = "";
    var $errorCode = -1;
    
    public function __construct($msg, $details, $errorCode) {
        parent::__construct($msg);
        $this->details = $details;
        $this->errorCode = $errorCode;
    }
}

class PbxConfig
{
    const presenceContact = "calendar:";
    var $ip = "";
    var $pw = "";
    var $account = "";
    var $dynPbx = "";
    var $ssl = false;
    var $port = "";
    var $domainFilter = "";
    var $domains = array();
    
    public function setDomainFilter($domainFilter)
    {
        $this->domainFilter = $domainFilter;
        $this->domains = explode(",", $domainFilter);
    }
    
    public function checkDomain($email)
    {
        if(!isset($this->domainFilter) || $this->domainFilter == "") return true;
        foreach($this->domains as $domain)
        {
            if(ends_with($email, $domain)) 
            {
                return true;
            }
        }
        return false;
    }
}

class EwsConfig
{
    /**
     * The microsoft exchange server
     *
     * @var string 
     *
     */    
    var $server = "";
    
    /**
     * Password of the accout
     *
     * @var string 
     *
     */    
    var $pw = "";
    
    /**
     * account, which is authorized for calendar view
     *
     * @var string 
     *
     */    
    var $account = "";
    
    /**
     * the StatusFrequency property gets or sets the frequency, specified in minutes, at which notification messages will be sent to the client when no events have occurred. 
     *
     * @var int 
     *
     */    
    var $statusFrequency = 10;
    
    /**
     * the timespan for which calendar entries are retrieved, in hours
     *
     * @var int 
     *
     */    
    var $availabilityTimespan = 36;
    
    /**
     * email recipients, who enable tentative calendar items
     *
     * @var string 
     *
     */    
    var $tentativeMails = "";
    
    /**
     *
     * @var string Exchange Version 
     */
    var $version = EWS_RequestServerVersion::Version_Exchange2010_SP2;
    
    /**
     * @var string
     */
    var $linuxIp = "";
    
    /**
     * @var string
     */
    var $linuxPort = "";
    
    /**
     * @var boolean
     */
    var $ssl = false;
    
    /**
     * @var boolean
     */
    var $omitSubject = false;
}

class ExchangeServer
{
    var $name = "";
    var $id = 0;
    var $desc = "";
    var $interval = 1;
    var $trace = false;
    var $error = "";
    var $lang = "en";
    /**
     *
     * @var PbxConfig
     */
    var $pbx = null;
    /**
     *
     * @var EwsConfig
     */
    var $ews = null;
    
    function ExchangeServer() {
        $this->pbx = new PbxConfig();
        $this->ews = new EwsConfig();
    }
    
    function read($id) {
        $this->id = $id;
        $exchangeDb = get_exchange_db();
        $query = "SELECT * FROM servers WHERE id=$id";
        $result = $exchangeDb->run($query);
        if(($row = pg_fetch_object($result))) {
            $this->name = $row->name;
            $this->desc = $row->description;
            $this->interval = $row->interval;
            $this->trace = ($row->trace == "t" ? true : false);
            $this->lang = $row->lang;
            $this->pbx->account = $row->pbx_account;
            $this->pbx->dynPbx = $row->pbx_dyn;
            $this->pbx->ip = $row->pbx_ip;
            $this->pbx->port = $row->pbx_port;
            $this->pbx->pw = $row->pbx_pw;
            $this->pbx->ssl = ($row->pbx_ssl == "t" ? true : false);
            $this->pbx->pw = rc4_decrypt(strrev($this->pbx->ip), $this->pbx->pw);
            $this->pbx->setDomainFilter($row->pbx_domain_filter);
            $this->ews->account = $row->ews_account;
            $this->ews->availabilityTimespan = $row->ews_availability_timespan;
            if($this->ews->availabilityTimespan < ExchangeAvailabilityTimespanField::minVal) {
                $this->ews->availabilityTimespan = ExchangeAvailabilityTimespanField::minVal;
            }
            $this->ews->pw = $row->ews_pw;
            $this->ews->server = $row->ews_server;
            $this->ews->statusFrequency = $row->ews_status_frequency;
            $this->ews->tentativeMails = $row->ews_tentative_mails;
            $this->ews->version = $row->ews_version;
            $this->ews->linuxIp = $row->ews_linux_ip;
            $this->ews->linuxPort = $row->ews_linux_port;
            $this->ews->pw = rc4_decrypt(strrev($this->ews->server), $this->ews->pw);
            $this->ews->ssl = ($row->ews_ssl == "t" ? true : false);
            $this->ews->omitSubject = ($row->ews_omit_subject == "t" ? true : false);
            return true;
        }
        else {
            $this->error = pg_last_error($exchangeDb->db);
        }
        return false;
    }
    
    function checkName($name) {
        $exchangeDb = get_exchange_db();
        $query = "SELECT id FROM servers WHERE name='".pg_escape_string($name)."' AND id!={$this->id};";
        $result = $exchangeDb->run($query);
        if(pg_num_rows($result) > 0) {
            $this->error = tl::tlx('A server with name {$arg[1]} already exists!', $name);
            return false;
        }
        return true;
    }
    
    function write() {
        $exchangeDb = get_exchange_db();
        if($exchangeDb->run("BEGIN") === false) {
            $this->error = pg_last_error($exchangeDb->db);
            return false;
        }
        $query = "LOCK TABLE servers, calendar_items, users IN EXCLUSIVE MODE";
        if($exchangeDb->run($query) === false) {
            $this->error = pg_last_error($exchangeDb->db);
            return false;
        }
        if($this->checkName($this->name) === false) {
            return false;
        }
        if($this->id > 0 && $this->delete() === false) {
            return false;
        }
        $idCol = ($this->id > 0 ? "id, " : "");
        $idVal = ($this->id > 0 ? "{$this->id}, " : "");
        $query = "INSERT INTO servers ($idCol name, description, interval, trace, lang, ".
                    "pbx_account, pbx_dyn, pbx_ip, pbx_port, pbx_pw, pbx_ssl, pbx_domain_filter, ".
                    "ews_account, ews_availability_timespan, ews_pw, ews_server, ews_linux_ip, ews_linux_port, ews_ssl, ews_omit_subject, ews_status_frequency, ews_tentative_mails, ews_version) VALUES ($idVal".
                    "'".pg_escape_string($this->name)."',".
                    "'".pg_escape_string($this->desc)."',".
                    "{$this->interval},".
                    ($this->trace ? "true" : "false").",".
                    "'".pg_escape_string($this->lang)."',".
                    "'".pg_escape_string($this->pbx->account)."',".
                    "'".pg_escape_string($this->pbx->dynPbx)."',".
                    "'".pg_escape_string($this->pbx->ip)."',".
                    "'".pg_escape_string($this->pbx->port)."',".
                    "'".pg_escape_string(rc4_encrypt(strrev($this->pbx->ip), $this->pbx->pw))."',".
                    ($this->pbx->ssl ? "true" : "false").",".
                    "'".pg_escape_string($this->pbx->domainFilter)."',".
                    "'".pg_escape_string($this->ews->account)."',".
                    "{$this->ews->availabilityTimespan},".
                    "'".pg_escape_string(rc4_encrypt(strrev($this->ews->server), $this->ews->pw))."',".
                    "'".pg_escape_string($this->ews->server)."',".
                    "'".pg_escape_string($this->ews->linuxIp)."',".
                    "'".pg_escape_string($this->ews->linuxPort)."',".
                    ($this->ews->ssl ? "true" : "false").",".
                    ($this->ews->omitSubject ? "true" : "false").",".
                    "{$this->ews->statusFrequency},".
                    "'".pg_escape_string($this->ews->tentativeMails)."',".
                    "'".pg_escape_string($this->ews->version)."') RETURNING id";
        $result = $exchangeDb->run($query);
        if($result === false || $exchangeDb->run("COMMIT") === false) {
            $this->error = pg_last_error($exchangeDb->db);
            return false;
        }
        if($this->id == 0) {
            if(($row = pg_fetch_object($result))) {
                $this->id = $row->id;
            }
        }
        return true;
    }
    
    function delete() {
        if($this->id == 0) return true;
        $exchangeDb = get_exchange_db();
        $oldServer = new ExchangeServer();
        if($oldServer->read($this->id) !== false) {
            $oldServer->clearPresence();
        }
        $query= "DELETE FROM calendar_items WHERE user_id IN (SELECT id FROM users WHERE server_id={$this->id});".
                "DELETE FROM users WHERE server_id={$this->id};".
                "DELETE FROM servers WHERE id={$this->id};";
        if($exchangeDb->run($query) === false) {
            $this->error = pg_last_error($exchangeDb->db);
            return false;
        }
        return true;
    }
    
    /**
     * 
     * @return boolean|\pbx
     */
    function pbxSoap() {
        $pbxSoap = null;
        try
        {
            $pbxSoap = new pbx($this->pbx->ip, $this->pbx->account, $this->pbx->pw, $this->pbx->ssl, $this->pbx->dynPbx, $this->pbx->port);
        }
        catch (Exception $e) {
            $pbxSoap = null;
            $this->error = tl::tl("PBX SOAP initialization failed with: ".$e->getMessage());
            return false;
        }
        return $pbxSoap;
    }
    
    /**
     * 
     * @return boolean|\ExchangeServices
     */
    function ewsSoap() {
        $ews = null;
        try
        {
            $ews = new ExchangeServices($this->ews->server, $this->ews->account, $this->ews->pw, $this->ews->version);
        }
        catch(Exception $e) {
            $this->error = tl::tl("Exchange Web Service initialization failed with: ".$e->getMessage());
            return false;
        }
        return $ews;
    }
    
    function clearPresence() {
        $ret = true;
        /* @var $pbxSoap Pbx */ $pbxSoap = $this->pbxSoap();
        if($pbxSoap === false) return false;
        
        $exchangeDb = get_exchange_db();
        $query = "SELECT DISTINCT u.guid, u.h323 FROM users u
                    JOIN calendar_items c ON u.id=c.user_id
                    WHERE u.server_id={$this->id}";
        $result = $exchangeDb->run($query);
        while(($row = pg_fetch_object($result))) {
            try
            {
                $presenceReturn = $pbxSoap->SetPresence(new PBX_Presence(), false, PbxConfig::presenceContact, $row->guid, null);
            }
            catch(Exception $e) {
                $ret = false;
                $this->error .= "PBX SetPresence for {$row->h323} failed with: ".$e->getMessage()."\n";
                continue;
            }
        }
        return $ret;
    }
}

class PbxUserStatus
{
    const del       = 0;
    const add       = 1;
    const modified  = 2;
    const up        = 3;
}

class PbxUser
{
    var $guid = "";
    var $id = 0;
    var $h323 = "";
    var $smtp = "";
    var $subscriptionId = "";
    var $subscriptionStatus = 0;
    var $availabilityStatus = 0;
    var $emails = "";
    var $exists = false;
    
    public function __construct($guid, $id, $h323, $emails, $smtp, $subscriptionId, $subscriptionStatus, $availabilityStatus) {
        $this->guid = $guid;
        $this->id = $id;
        $this->h323 = $h323;
        $this->smtp = (!$smtp ? "" : $smtp);
        $this->emails = $emails;
        $this->subscriptionId = (!$subscriptionId ? "" : $subscriptionId);
        $this->subscriptionStatus = (!$subscriptionStatus ? 0 : $subscriptionStatus);
        $this->availabilityStatus = (!$availabilityStatus ? 0 : $availabilityStatus);
    }
}

class ExchangeConfigConsts
{
    const pageConfig = 1;
    const tabPgsql = "pgsql", tabServer = "server";
    const modeServer = "modeserver", modePbx = "modepbx", modeEws = "modeews", modeDeleteServer = "modedelete";
    const modeInRePgPw = "inrepgpw";
}

abstract class ExchangeInputPage extends MainInnovaphonePage 
{
    public function ExchangeInputPage($site, $name, $printMenu = true) {
        $this->enableTranslations = true;
        parent::MainInnovaphonePage($site, $name, $printMenu);
        $this->logoImg = "/apps/innovaphone-exchange/images/logo_exchange.png";
        // add java script file
        $this->scriptfiles += array("javaex" => "/apps/innovaphone-exchange/java.js");
        
        //$this->links[count($this->links)] = array("rel" => "stylesheet", "type" => "text/css", "href" => "style_reporting.css");        
        
        // default pageno if not set (1)
        if (($this->pageNo = $this->page()) == "") $this->pageNo = 1;        
        $version = 0;
        $hotfix = 0;
        MainInputPage::getVersions(Config::webRoot, $version, $hotfix);
        global $exchangeSession;
        $app = $exchangeSession->mainSession->getApp("innovaphone-exchange");
        if(((int)$app->hotfix + (int)$app->hotfixOffset) < (int)$hotfix) {
            die(tl::tlx('Install innovaphone-exchange hotfix{$arg[1]} first!', $hotfix));
            exit;
        }
        $this->searchArea = "<br><br><b>{$app->versionStr}</b>";
    }
    
    function addMenu() {
        $this->addMainMenuItem(tl::tl("Config"), "config_exchange.php", $this->name == "exchangeconfig");
        $this->addMainMenuItem(tl::tl("Diagnostics"), "diagnostics.php", $this->name == "innovaphonediagnostics");
    }    
    
    function getHelpLink() {
        return Config::wikiHelpExchange.$this->pages[$this->pageNo]->getHelpAnchor();
    }
    
    public function tabDbExchange(MyPage &$page) {
        $page->currentSection = InnovaphoneDiagnosticsConsts::tabDbExchange;
        $options = array();
        $exchangeDb = get_exchange_db();
        $query = "SELECT name, id FROM SERVERS ORDER BY LOWER(name)";
        $result = $exchangeDb->run($query);
        $options[0] = tl::tl("All Servers");
        while(($row = pg_fetch_object($result))) {
            $options[$row->id] = $row->name;
        }
        
        $page->addField($drop = new InputPageDropdownField("servers", tl::tl("Server"), $options));
        $drop->default[0] = true;
        $page->addField($user = new InputPageText("users", $page->getModeLink(InnovaphoneDiagnosticsConsts::modeExchangeUsers, tl::tl("List Users"))));
        $user->fullline = true;
        $user->tdattributes["style"] = "padding-top: 10px;";
        $cal = $page->addField(new InputPageText("cal", $page->getModeLink(InnovaphoneDiagnosticsConsts::modeExchangeItems, tl::tl("List Calendar Items"))));
        $cal->fullline = true;
        
        switch($this->currentMode) {
            case InnovaphoneDiagnosticsConsts::modeExchangeUsers:
                $this->addModeExchangeUsers($page);
                break;
            case InnovaphoneDiagnosticsConsts::modeExchangeItems:
                $this->addModeExchangeItems($page);
                break;
        }
    }
    
    public function addModeExchangeUsers(MyPage &$page) {
        $page->currentSection = $this->currentMode;
        $page->addSection($this->currentMode, 1, true, tl::tl("Users"));
        $list = $page->addField(new MainInputPageListField("listusers"));
        $emptyText = tl::tl("No users");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable",
            "<b>$emptyText</b>");
        $empty->style = $page->currentSection;
        $list->addColumn($server = new InputPageText($list->name."server", null), tl::tl("Server"));
        $server->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $h323 = $list->addColumn( new InputPageText($list->name."h323", null), "H323");
        $h323->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $smtp = $list->addColumn( new InputPageText($list->name."smtp", null), tl::tl("Primary SMTP address"));
        $smtp->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $subs = $list->addColumn(new InputPageText($list->name."subs", null), tl::tl("Subscription Status"));
        $subs->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $av = $list->addColumn(new InputPageText($list->name."av", null), tl::tl("Availability Status"));
        $av->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($mails = new InputPageText($list->name."mails", null), tl::tl("PBX mail addresses"));
        $mails->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
    }
    
    public function addModeExchangeItems(MyPage &$page) {
        $page->currentSection = $this->currentMode;
        $page->addSection($this->currentMode, 1, true, tl::tl("Calendar Items"));
        $list = $page->addField(new MainInputPageListField("listitems"));
        $emptyText = tl::tl("No calendar items");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable",
            "<b>$emptyText</b>");
        $empty->style = $page->currentSection;
        $list->addColumn($server = new InputPageText($list->name."server", null), tl::tl("Server"));
        $server->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $h323 = $list->addColumn( new InputPageText($list->name."h323", null), "H323");
        $h323->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $start = $list->addColumn( new InputPageText($list->name."start", null), tl::tl(/*Calendar entry start time*/"From"));
        $start->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $end = $list->addColumn(new InputPageText($list->name."end", null), tl::tl(/*Calendar entry end time*/"To"));
        $end->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $meeting = $list->addColumn(new InputPageText($list->name."meeting", null), tl::tl("Meeting?"));
        $meeting->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $private = $list->addColumn(new InputPageText($list->name."private", null), tl::tl("Private?"));
        $private->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $busy = $list->addColumn(new InputPageText($list->name."busy", null), tl::tl("Busy Type"));
        $busy->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $pbx = $list->addColumn(new InputPageText($list->name."pbx", null), tl::tl("PBX Status"));
        $pbx->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $subject = $list->addColumn(new InputPageText($list->name."subject", null), tl::tl("Subject"));
        $subject->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $location = $list->addColumn(new InputPageText($list->name."location", null), tl::tl("Location"));
        $location->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
    }
    
    public function fillListExchange() {
        switch($this->currentMode) {
            case InnovaphoneDiagnosticsConsts::modeExchangeUsers:
                $this->fillListExchangeUsers();
                break;
            case InnovaphoneDiagnosticsConsts::modeExchangeItems:
                $this->fillListExchangeItems();
                break;
        }
    }
    
    public function fillListExchangeUsers() {
        $list = $this->getField("listusers");
        $server = $this->getField("servers");
        $id = array_search(true, $server->value);
        $exchangeDb = get_exchange_db();
        $query = "SELECT u.h323, u.primary_smtp, u.subscription_status, u.availability_status, u.emails, s.name
                    FROM users u
                    JOIN servers s ON u.server_id=s.id";
        if($id != 0) {
            $query .= " WHERE s.id=$id";
        }
        $result = $exchangeDb->run($query);
        $list->value = array();
        $rows = 0;
        while(($row = pg_fetch_object($result))) {
            $list->value[$rows][$list->name."server"] = htmlentities_utf8($row->name);
            $list->value[$rows][$list->name."h323"] = htmlentities_utf8($row->h323);
            $list->value[$rows][$list->name."smtp"] = htmlentities_utf8($row->primary_smtp);
            if($row->subscription_status) {
                $time = strtotime($row->subscription_status);
                $list->value[$rows][$list->name."subs"] = InputPageDateField::translateDate($time).date(" H:i:s", $time);
            }
            else {
                $list->value[$rows][$list->name."subs"] = "N/A";
            }
            if($row->availability_status) {
                $time = strtotime($row->availability_status);
                $list->value[$rows][$list->name."av"] = InputPageDateField::translateDate($time).date(" H:i:s", $time);
            }
            else {
                $list->value[$rows][$list->name."av"] = "N/A";
            }
            $list->value[$rows][$list->name."mails"] = $row->emails;
            $rows++;
        }
    }
    
    public function fillListExchangeItems() {
        $list = $this->getField("listitems");
        $server = $this->getField("servers");
        $id = array_search(true, $server->value);
        $exchangeDb = get_exchange_db();
        $query = "SELECT u.h323, c.subject, c.location, c.time_end, c.time_start, c.is_meeting, c.is_private, c.busy_type, c.pbx_status, s.name
                    FROM calendar_items c
                    JOIN users u ON (c.user_id=u.id)
                    JOIN servers s ON u.server_id=s.id";
        if($id != 0) {
            $query .= " WHERE s.id=$id";
        }
        $query .= " ORDER BY c.time_start";
        $result = $exchangeDb->run($query);
        $list->value = array();
        $rows = 0;
        while(($row = pg_fetch_object($result))) {
            $list->value[$rows][$list->name."server"] = htmlentities_utf8($row->name);
            $list->value[$rows][$list->name."h323"] = htmlentities_utf8($row->h323);
            $list->value[$rows][$list->name."subject"] = htmlentities_utf8($row->subject);
            $list->value[$rows][$list->name."location"] = htmlentities_utf8($row->location);
            $time = strtotime($row->time_start);
            $list->value[$rows][$list->name."start"] = InputPageDateField::translateDate($time).date(" H:i:s", $time);
            $time = strtotime($row->time_end);
            $list->value[$rows][$list->name."end"] = InputPageDateField::translateDate($time).date(" H:i:s", $time);
            $list->value[$rows][$list->name."busy"] = $row->busy_type;
            $list->value[$rows][$list->name."pbx"] = $row->pbx_status;
            $list->value[$rows][$list->name."meeting"] = ($row->is_meeting == "t" ? tl::tl("Yes") : tl::tl("No"));
            $list->value[$rows][$list->name."private"] = ($row->is_private == "t" ? tl::tl("Yes") : tl::tl("No"));
            $rows++;
        }
    }
}

class ExchangeStatusFrequencyField extends InputPageDigitStringField {
    const minVal = 1;
    const maxVal = 1440;
    
    public function valid() {
        if(!parent::valid()) return false;
        $value = intval($this->value);
        if($value < self::minVal || $value > self::maxVal) {
            $this->msg = tl::tlx('The status frequency must be between {$arg[1]} and {$arg[2]} minutes.', self::minVal, self::maxVal);
            return false;
        }
        return true;
    }
}

class PbxPresenceUpdateField extends InputPageDigitStringField {
    const minVal = 1;
    const maxVal = 59;
    
    public function valid() {
        if(!parent::valid()) return false;
        $value = intval($this->value);
        if($value < self::minVal || $value > self::maxVal) {
            $this->msg = tl::tlx('The presence update interval must be between {$arg[1]} and {$arg[2]} minutes.', self::minVal, self::maxVal);
            return false;
        }
        return true;
    }
}

class ExchangeAvailabilityTimespanField extends InputPageDigitStringField {
    const minVal = 16;  // 16 hours and calendar entries are retrieved 12 hours before the timespan ends
    const maxVal = 72;
    
    public function valid() {
        if(!parent::valid()) return false;
        $value = intval($this->value);
        if($value < self::minVal || $value > self::maxVal) {
            $this->msg = tl::tlx('The availability timespan must be between {$arg[1]} and {$arg[2]} hours.', self::minVal, self::maxVal);
            return false;
        }
        return true;
    }
}

class ExchangeXmlCfg {
    var $error = "";
    var $mtime = 0;
    
    /*
     * @var integer timestamp of last presence update
     */
    var $lastPresenceUpdate = null;
    
    public function ExchangeXmlCfg() {
        $this->readConfig();
    }
    
    public function readConfig($file = Config::exchangeConfig, $import = false) {
        if(!$import) {
            
        }
        $doc = new DOMDocument();
        $doc->encoding = "utf-8";
        $doc->preserveWhiteSpace = false;
        $this->pbx = new PbxConfig();
        $this->ews = new EwsConfig();
        @$doc->load($file);
        if($doc !== false) {
            if(!$import && file_exists($file)) {
                $this->mtime = filemtime($file);
            }
            $config = $doc->getElementsByTagName ("config");
            if($config->length > 0) {
                $config = $config->item(0);
                foreach($config->childNodes as $node) {
                    switch($node->nodeName) {
                        case "last-presence-date-update":
                            $this->lastPresenceUpdate = intval($node->textContent); break;
                            break;
                        default:
                            break;
                    }
                }
            }
            else {
                $this->error = tl::tlx('Config file {$arg[1]} does not contain a <config> tag.', $file);
                return false;
            }
        }
        else {
            $this->error = tl::tlx('Couldn\'t parse config file {$arg[1]}.', $file);
            return false;
        }
        return true;
    }
    
    public function writeConfig() {
        $doc = new DOMDocument("1.0", "utf-8");
        $doc->formatOutput = true;
        $config = $doc->appendChild($doc->createElement("config"));
        
        // global stuff
        $config->appendChild($doc->createElement("last-presence-date-update", $this->lastPresenceUpdate));
        
        // save doc
        if($doc->save(Config::exchangeConfig) === false) {
            $this->error = tl::tlx('Couldn\'t rewrite exchange config \'{$arg[1]}\'.', Config::exchangeConfig);
            return false;
        }
    }
    
    public function write() {
        if($this->writeConfig() === false) {
            return false;
        }
        
        global $exchangeSession;
        $exchangeSession->updateExchangeCfg($this);
        return true;
    }
}

class ExchangeSession {
    /**
     * ExchangeConfig
     *
     * @var ExchangeXmlConfig 
     *
     */    
    var $exchangeCfg = null;
    
    /**
     * @var ExchangeServer
    */
    var $exServer = null;
    
    var $mainSession;
    
    public function ExchangeSession() {
        global $mainSession;
        $this->mainSession = $mainSession;
        $this->exServer = new ExchangeServer();
        if($this->mainSession->restore($this->exchangeCfg, "exchangecfg", false) === false) {
            $this->exchangeCfg = new ExchangeXmlCfg();
            $this->mainSession->save($this->exchangeCfg, "exchangecfg");
            $this->mainSession->save($this->exServer, "exchangeserver");
        }
        $this->mainSession->restore($this->exchangeCfg, "exchangecfg");
        $this->mainSession->restore($this->exServer, "exchangeserver");
        $this->checkExchangeCfg();
    }
    
    function checkExchangeCfg() {
        if(file_exists(Config::exchangeConfig)) {
            $mtime = filemtime(Config::exchangeConfig);
            if($this->exchangeCfg == null || $mtime != $this->exchangeCfg->mtime) {
                $this->updateExchangeCfg(new ExchangeXmlCfg());
            }
        }
    }
    
    function updateExchangeCfg($cfg) {
        $this->exchangeCfg = $cfg;
        $this->mainSession->save($this->exchangeCfg, "exchangecfg");
    }
    
    function updateExchangeServer($server) {
        $this->exServer = $server;
        $this->mainSession->save($this->exServer, "exchangeserver");
    }
}

class ExchangeMethods {
    static function getAlarmServer() {
        /* @var $exchangeSession ExchangeSession */ global $exchangeSession;
        $alarmServer = new AlarmServer();
        if($exchangeSession->exchangeCfg->pbx->ip == "") {
            return $alarmServer;
        }
        $alarmServer->ip = $exchangeSession->exchangeCfg->pbx->ip;
        $alarmServer->user = $exchangeSession->exchangeCfg->pbx->account;
        $alarmServer->pw = $exchangeSession->exchangeCfg->pbx->pw;
        $alarmServer->https = $exchangeSession->exchangeCfg->pbx->ssl;
        return $alarmServer;
    }
}

$exchangeDb = null;
function get_exchange_db() {
    global $exchangeDb;
    if ($exchangeDb == null) {
        $exchangeDb = new DbPGSql("innovaphone-exchange", "innovaphone-exchange", "", "config_exchange.php");
        if (!$exchangeDb->isOpen) {
            send_alarm("No Database available", "Return error by Connect", Alarms::alExchangeConnectDb, false, ExchangeMethods::getAlarmServer());
            home_die(tl::tl("No postgresql db available."), "config_exchange.php");
            exit;
        }
        else {
            clear_alarm(Alarms::alExchangeConnectDb, ExchangeMethods::getAlarmServer());
        }
    }
    return $exchangeDb;
}

function close_exchange_db() {
    global $exchangeDb;
    if($exchangeDb) {
        $exchangeDb->close();
        $exchangeDb = null;
    }
}

/** @var ExchangeSession */$exchangeSession = new ExchangeSession();
if($exchangeSession->mainSession->error) {
    $exchangeSession->mainSession->destroy();
    home_die($exchangeSession->mainSession->error, "config_exchange.php");
}

?>